
# coding:utf-8

import requests
import pandas as pd
from sqlalchemy import create_engine, text
from datetime import datetime
from tqdm import tqdm
from config import XUEQIU_HEADERS
import gc
import time

class StockDailyDataCollector:
    """股票日线数据采集器类"""

    def __init__(self, db_url):
        self.engine = create_engine(
            db_url,
            pool_size=5,
            max_overflow=10,
            pool_recycle=3600
        )
        self.headers = XUEQIU_HEADERS

    def fetch_all_stock_codes(self):
        query = "SELECT gp_code FROM gp_code_all"
        df = pd.read_sql(query, self.engine)
        return df['gp_code'].tolist()

    def fetch_daily_stock_data(self, symbol, begin):
        url = f"https://stock.xueqiu.com/v5/stock/chart/kline.json?symbol={symbol}&begin={begin}&period=day&type=before&count=-1&indicator=kline,pe,pb,ps,pcf,market_capital,agt,ggt,balance"
        try:
            response = requests.get(url, headers=self.headers, timeout=10)
            return response.json()
        except Exception as e:
            print(f"Request error for {symbol}: {e}")
            return {'error_code': -1, 'error_description': str(e)}

    def transform_data(self, data, symbol):
        try:
            items = data['data']['item']
            columns = data['data']['column']
        except KeyError as e:
            print(f"KeyError for {symbol}: {e}")
            return None

        df = pd.DataFrame(items, columns=columns)
        df['symbol'] = symbol

        required_columns = ['timestamp', 'volume', 'open', 'high', 'low', 'close',
                            'chg', 'percent', 'turnoverrate', 'amount', 'symbol', 'pb', 'pe', 'ps']
        existing_columns = [col for col in required_columns if col in df.columns]
        df = df[existing_columns]

        if 'timestamp' in df.columns:
            df['timestamp'] = pd.to_datetime(df['timestamp'], unit='ms', utc=True).dt.tz_convert('Asia/Shanghai')

        return df

    def save_batch_to_database(self, batch):
        if batch:
            df_all = pd.concat(batch, ignore_index=True)
            df_all.to_sql('gp_day_data', self.engine, if_exists='append', index=False)

    def fetch_data_for_date(self, date=None):
        if date is None:
            start_date = datetime.now()
            date_str = start_date.strftime('%Y-%m-%d')
        else:
            start_date = datetime.strptime(date, '%Y-%m-%d')
            date_str = date

        delete_query = text("DELETE FROM gp_day_data WHERE `timestamp` LIKE :date_str")
        with self.engine.begin() as conn:
            conn.execute(delete_query, {"date_str": f"{date_str}%"})

        stock_codes = self.fetch_all_stock_codes()
        begin = int(start_date.replace(hour=0, minute=0, second=0, microsecond=0).timestamp() * 1000)

        batch_data = []
        for idx, symbol in enumerate(tqdm(stock_codes, desc=f"Fetching and saving daily stock data for {date_str}")):
            data = self.fetch_daily_stock_data(symbol, begin)

            if data.get('error_code') == 0:
                df = self.transform_data(data, symbol)
                if df is not None:
                    batch_data.append(df)
            else:
                print(f"Error fetching data for {symbol} on {date_str}: {data.get('error_description')}")

            if len(batch_data) >= 100:
                self.save_batch_to_database(batch_data)
                batch_data.clear()
                gc.collect()

        # Save remaining data
        if batch_data:
            self.save_batch_to_database(batch_data)
            gc.collect()

        self.engine.dispose()
        print(f"Daily data fetching and saving completed for {date_str}.")

def collect_stock_daily_data(db_url, date=None):
    collector = StockDailyDataCollector(db_url)
    collector.fetch_data_for_date(date)

if __name__ == "__main__":
    db_url = 'mysql+pymysql://root:Chlry#$.8@192.168.18.199:3306/db_gp_cj'
    collect_stock_daily_data(db_url)
